# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
# GPL 2008-2010
from __future__ import division
import Image
import time
import math
from struct import unpack

import numpy as np
import gobject
import gst


class Audio(gst.Pipeline):
    volume  = []
    position = 0

    def __init__(self, uri, channels=2):
        gst.Pipeline.__init__(self)
        self.duration = -1
        self.framerate = 25
        self.channels = channels

        self.uri = uri

        self.src = gst.element_factory_make("filesrc")
        self.src.props.location = self.uri
        self.sbin = gst.element_factory_make("decodebin2")
        self.sbin.props.caps = gst.Caps("audio/x-raw-int;audio/x-raw-float")
        self.sbin.props.expose_all_streams = False

        self.queue = gst.element_factory_make("queue")
        self.rate = gst.element_factory_make("audioresample")
        self.conv = gst.element_factory_make("audioconvert")
        self.level = gst.element_factory_make("level")
        self.level.props.interval = int(gst.SECOND / self.framerate)

        self.sink = gst.element_factory_make("fakesink")

        self.add(self.src, self.sbin, self.conv, self.queue, self.rate, self.level, self.sink)


        self.src.link(self.sbin)
        self.sbin.connect('pad-added', self._sbinPadAddedCb)

        self.set_state(gst.STATE_PAUSED)
        self.get_state()
        self.getDuration()
        self.frames = int((float(self.duration) / gst.SECOND) * float(self.framerate))

    def _sbinPadAddedCb(self, unused_sbin, pad):
        caps = pad.get_caps()
        if 'audio' in str(caps):
            self.samplerate = caps[0]['rate']
            if not isinstance(self.samplerate, int):
                self.samplerate = self.samplerate.high
            pad.link(self.queue.get_pad("sink"))
            self.queue.link(self.conv)
            self.conv.link(self.rate,
                           gst.Caps("audio/x-raw-int,channels=%s,width=16,depth=16" %
                                                            (self.channels, )))
            self.rate.link(self.level,
                           gst.Caps("audio/x-raw-int,rate=%s,channels=%s,width=16,depth=16" %
                                                            (self.samplerate, self.channels)))
            self.level.link(self.sink)

    def getDuration(self):
        if self.duration < 0:
            pads = self.sink.sink_pads()
            q = gst.query_new_duration(gst.FORMAT_TIME)
            for pad in pads:
                if pad.get_peer() and pad.get_peer().query(q):
                  format, self.duration = q.parse_duration()
        return self.duration

    def done(self):
        pass

    def onBusMessage(self, bus, message):
        if message.src == self.level:
            struct = message.structure
            if struct.get_name() == 'level':
                self.volume.append((
                    pow (10, struct['rms'][0] / 20),
                    pow (10, struct['rms'][1] / 20)
                ))

        if message.src == self and message.type == gst.MESSAGE_EOS:
            self.done()

class Timeline(Audio):
    def __init__(self, uri, prefix, width, height, mode='waveform'):
        Audio.__init__(self, uri)
        self.mode = mode

        bus = self.get_bus()
        bus.add_signal_watch()
        self.watch_id = bus.connect("message", self.onBusMessage)

        self.mainloop = gobject.MainLoop()

        self.tile_width = width
        self.tile_height = height
        if self.mode == 'pixel':
            self.tile_height = self.channels
        self.prefix = prefix
        self.timeline_fps = 25
        self.input_tile_width = int(math.ceil((float(self.framerate)/self.timeline_fps) * width))
        ntiles = int(math.ceil(float(self.frames)/self.input_tile_width))
        self.tiles = []
        for i in range(ntiles):
            tile = Image.new("RGB", (self.input_tile_width, height), (0, 0, 0))
            self.tiles.append(tile)
        
        self.set_state(gst.STATE_PLAYING)
        self.mainloop.run()

        if self.mode == 'pixel':
            for volume in self.volume:
                tile = int(math.floor(float(self.position) / self.input_tile_width))
                tilePos = int(self.position - (tile * self.input_tile_width))
                for c in range(0, self.channels):
                    color = tuple(3 * [int(volume[c]*255)])
                    self.tiles[tile].putpixel((tilePos, c), color)
                self.position += 1
        else:
            normalize = 1/max([max(*v) for v in self.volume])
            for volume in self.volume:
                tile = int(math.floor(float(self.position) / self.input_tile_width))
                tilePos = int(self.position - (tile * self.input_tile_width))

                def plot(p, start):
                    height = normalize * p * self.tile_height
                    crop = int((self.tile_height-height) / 2)
                    color = (p, p, p)
                    color = (255,255,255)

                    if start:
                        end = self.tile_height - crop
                    else:
                        start = crop
                        end = int(self.tile_height/2)
                    for i in range(start, end):
                        self.tiles[tile].putpixel((tilePos, i), color)
                #left
                plot(volume[0], 0)
                #right
                plot(volume[1], int(self.tile_height/2))

                self.position += 1

        for  i in range(ntiles):
            tile = self.tiles[i]
            if tile.size[0] != self.tile_width:
                tile = tile.resize((self.tile_width, self.tile_height), Image.ANTIALIAS)
            if i < (ntiles-1):
                frames = self.input_tile_width
            else:
                frames = self.frames-((ntiles-1)*self.input_tile_width)
            tile_width = int(math.ceil(self.timeline_fps*frames)/float(self.framerate))
            if -2 < self.tile_width - tile_width < 2:
                tile_width = self.tile_width
            tile = tile.crop((0, 0, tile_width, self.tile_height))
            filename = "%s%sp%04d.png" % (self.prefix, self.tile_height, i)
            tile.save(filename)

    def done(self):
        self.mainloop.quit()
