# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
# GPL 2008-2010
from __future__ import division

import Image
import math

import gobject
import gst

from imagesink import ImageSink


class Video(gst.Pipeline):
    duration = -1

    def __init__(self, uri, height=64, framerate=gst.Fraction(25, 1)):
        gst.Pipeline.__init__(self)
        self.height = height
        self.framerate = framerate

        self.src = gst.element_factory_make("filesrc")
        self.src.props.location = uri
        self.sbin = gst.element_factory_make("decodebin2")
        self.sbin.props.caps = gst.Caps("video/x-raw-yuv;video/x-raw-rgb")
        self.sbin.props.expose_all_streams = False
        
        self.csp = gst.element_factory_make("ffmpegcolorspace")
        self.scale = gst.element_factory_make("videoscale")
        self.rate = gst.element_factory_make("videorate")
        self.queue = gst.element_factory_make("queue")

        self.sink = ImageSink()
        self.sink.connect('frame', self._frameCb)

        self.add(self.src, self.sbin, self.csp, self.queue, self.scale, self.rate, self.sink)

        self.src.link(self.sbin)
        self.sbin.connect('pad-added', self._sbinPadAddedCb)

        self.set_state(gst.STATE_PAUSED)
        self.get_state()
        self.getDuration()
        self.frames = int((float(self.duration) / gst.SECOND) * float(self.framerate))

    def _sbinPadAddedCb(self, unused_sbin, pad):
        caps = pad.get_caps()
        if 'video' in str(caps):
            pad.link(self.queue.get_pad("sink"))
            self.queue.link(self.scale)
            self.scale.link(self.rate)
            height = self.height
            if 'width' in caps[0].keys() and isinstance(caps[0]['width'], int):
                width = int(caps[0]['width']/caps[0]['height'] * height)
            else:
                width = int(4/3 * height)
            if width % 4:
                width += 4 - width % 4
            if height % 4:
                height += 4 - height % 4

            self.rate.link(self.csp, gst.Caps("video/x-raw-rgb;video/x-raw-yuv,framerate=%s/%s,width=%s,height=%s" % (self.framerate.num, self.framerate.denom, width, height)))
            self.csp.link(self.sink)

    def _frameCb(self, unused_thsink, frame, timestamp):
        pass

    def getDuration(self):
        if self.duration < 0:
            pads = self.sink.sink_pads()
            q = gst.query_new_duration(gst.FORMAT_TIME)
            for pad in pads:
                if pad.get_peer() and pad.get_peer().query(q):
                  format, self.duration = q.parse_duration()
        return self.duration

class Timeline(Video):
    _ready = False
    lastPos=0
    timeline_fps = 25
    mode = 'average'

    def __init__(self, uri, prefix, width, height, mode):
        Video.__init__(self, uri, height, gst.Fraction(self.timeline_fps, 1))
        self.mode = mode
        bus = self.get_bus()
        bus.add_signal_watch()
        self.watch_id = bus.connect("message", self.onBusMessage)

        self.mainloop = gobject.MainLoop()

        self.tile_width = width
        self.tile_height = height
        self.prefix = prefix
        self.input_tile_width = int(math.ceil((float(self.framerate)/self.timeline_fps) * width))
        ntiles = int(math.ceil(float(self.frames)/self.input_tile_width))
        self.tiles = []
        for i in range(ntiles):
            tile = Image.new("RGB", (self.input_tile_width, height))
            self.tiles.append(tile)
        
        self.set_state(gst.STATE_PLAYING)
        self.mainloop.run()

        for  i in range(ntiles):
            tile = self.tiles[i]
            if tile.size[0] != self.tile_width:
                tile = tile.resize((self.tile_width, self.tile_height), Image.ANTIALIAS)
            if i < (ntiles-1):
                frames = self.input_tile_width
            else:
                frames = self.frames-((ntiles-1)*self.input_tile_width)
            tile_width = int(math.ceil(self.timeline_fps*frames)/float(self.framerate))
            if -2 < self.tile_width - tile_width < 2:
                tile_width = self.tile_width
            tile = tile.crop((0, 0, tile_width, self.tile_height))
            filename = "%s%sp%04d.png" % (self.prefix, self.tile_height, i)
            tile.save(filename)

    def done(self):
        self.mainloop.quit()

    def _frameCb(self, unused_thsink, frame, timestamp):
        if not self._ready:
            # we know we're prerolled when we get the initial thumbnail
            self._ready = True
        else:
            _framePos = int(math.ceil((float(timestamp) / (gst.SECOND) * float(self.framerate)))) + 1
            if self.mode == 'center':
                frame = frame.crop((int(frame.size[1]/2), 0,
                                    int(frame.size[1]/2)+1, frame.size[0]))
                frame = frame.resize((1, self.tile_height), Image.ANTIALIAS)
            else:
                frame = frame.resize((1, self.tile_height), Image.ANTIALIAS)

            for framePos in range(self.lastPos, _framePos):
                tile = int(math.floor(float(framePos) / self.input_tile_width))
                tilePos = framePos - (tile * self.input_tile_width)
                for i in range(self.tile_height):
                    self.tiles[tile].putpixel((tilePos, i), frame.getpixel((0, i)))

            self.lastPos = _framePos
            if self.mainloop and timestamp >= self.duration:
                self.done()

    def onBusMessage(self, bus, message):
        if message.src == self and message.type == gst.MESSAGE_EOS:
            self.done()

